<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 * This is the debug module. It's used widely accross the framework to log data and allow data feedback on a separate channel (backend).
 *
 * It supports level and domain filtering.
 *
 * Currently supported backend:
 * - server log: the data is written to the error_log
 * - firePHP: the data is send with the webpage to be displayed by FirePHP
 */
/**
 * Adds FirePHP support to PHPDevShell.
 *
 * @author Greg
 */
class PHPDS_debug extends PHPDS_dependant
{
	/* levels of verbosity: the higher, the more data will be sent */
	const DEBUG = 4;
	const INFO = 3;
	const WARN = 2;
	const ERROR = 1;
	const LOG = 0;
	protected $enabled = false; /* boolean: is the data to be sent, anyway */
	protected $level = PHPDS_debug::LOG; /* level of verbosity */
	protected $domain; /* string (maybe null): to which semantic domain this instance is related */
	protected $conduits = null;

	public function __construct ($dependance, $domain = null)
	{
		$this->PHPDS_dependance($dependance);
		$this->conduits = $this->errorHandler;
		if (isset($this->configuration['debug'])) {
			$configuration = $this->configuration['debug'];
			$this->enabled = !empty($configuration['enable']);
			$this->level = empty($configuration['level']) ? PHPDS_debug::LOG : intval($configuration['level']);
		} else {
			$this->enabled = false;
		}
		$this->domain(empty($domain) ? '!' : $domain);
	}

	/**
	 * Accessor for the domain field: get (and possibily set) the domain
	 *
	 * The domain is the semantic field of the data sent to this instance ; it's used to filter which data will be actually sent
	 * Note: the object will enable/disabled itself based on the domain debug configuration: the domain MUST be in the array to be active.
	 *
	 * @param $domain string (optional) the semantic domain
	 * @return string, the domain (maybe null)
	 */
	public function domain ($domain = null)
	{
		if (! is_null($domain)) {
			$this->domain = $domain;
			if (isset($this->configuration['debug'])) {
				$configuration = $this->configuration['debug'];
				if (isset($configuration['domains'])) {
					if (is_array($configuration['domains'])) {
						$this->enabled = in_array($domain, $configuration['domains']);
					}
				}
			}
		}
		return $domain;
	}

	/**
	 * Enable or disable the debugger output ; get the current state
	 *
	 * Note: at this time, the debugger has to be enabled at startup
	 *
	 * @param $doit				(optional) enable (true or disable (false)
	 * @return boolean		weither it's currently enabled
	 */
	function enable($doit = null)
	{
		if (!is_null($doit)) $this->enabled = (boolean)$doit;
		return $this->enabled;
	}

	/**
	 * Is this instance sending data?
	 *
	 * @return boolean
	 */
	public function isEnabled()
	{
		return ($this->enabled == true);
	}

	/**
	 * Magic method: shortcut to log($ata)
	 */
	public function __invoke($data, $label = null)
	{
		return $this->log($data);
	}

	/**
	 * Dump the content of a variable to the backends
	 *
	 * @param $data
	 * @param $label
	 */
	public function dump($data, $label = 'data')
	{
		if (!$this->enabled) return;

		if ($this->firephp) $this->firephp->dump($label, $data);
		$this->error_log('DUMP', $data);
	}

	/**
	 * Log the data to the backends with the LOG level (the smallest, most often seen)
	 *
	 * @param $data
	 * @param $label
	 */
	public function log($data, $label = null)
	{
		if (!$this->enabled || ($this->level < PHPDS_debug::LOG)) return;

		if (empty($label)) $label = $this->domain;


		$this->conduits->conductor($data, PHPDS_debug::LOG, $label);
	}
	/**
	 * Push Firebug Debug Info
	 *
	 * @param mixed $data
	 * @return void
	 */
	public function debug($data)
	{
		if (!$this->enabled || ($this->level < PHPDS_debug::DEBUG)) return;

		$this->conduits->conductor($data, PHPDS_debug::DEBUG);
	}
	/**
	 * Push Firebug Info
	 *
	 * @param mixed $data
	 * @return void
	 */
	public function info($data)
	{
		if (!$this->enabled || ($this->level < PHPDS_debug::INFO)) return;

		$this->conduits->conductor($data, PHPDS_debug::INFO);
	}
	/**
	 * Push Firebug Warning
	 *
	 * @param mixed $data
	 * @return void
	 */
	public function warn($data)
	{
		if (!$this->enabled || ($this->level < PHPDS_debug::WARN)) return;

		$this->conduits->conductor($data, PHPDS_debug::WARN);
	}
	/**
	 * Push Firebug Warning
	 *
	 * @param mixed $data
	 */
	public function warning($data)
	{
		return $this->warn($data);
	}
	/**
	 * Push Firebug Error
	 *
	 * @param mixed $data
	 * @return void
	 */
	public function error($datal)
	{
		if (!$this->enabled || ($this->level < PHPDS_debug::ERROR)) return;

		$this->conduits->conductor($data, PHPDS_debug::ERROR);
	}
	/**
	 * FirePHP Process
	 *
	 * @param mixed $data
	 * @return void
	 */
	public function process($data)
	{
		$content = $data['sql'];
		$key = '['.$data['counter'].'] '.$data['key'].' ('.$data['result'].')';

		$this->log($content, $key);
	}

}

?>